'use strict';

const https = require('https');

const RUNTIME = 'nodejs4.3';
const TIMEOUT_MS = 500;
const ENDPOINTS = {
  'us-west-2': 'aws-us-west-2.serverlessapi.com',
  'us-east-1': 'aws-us-east-1.serverlessapi.com',
};

let coldStart = true;

class SDK {
  constructor(APIKey, options) {
    this.APIKey = APIKey;

    this.requestOptions = {
      host: ENDPOINTS[process.env.AWS_REGION],
      path: '/v0/events',
      method: 'POST',
    };

    if (options && options.endpoint) {
      this.requestOptions.host = options.endpoint;
    }
  }

  sendInvocation(invocation, callback) {
    const tags = invocation.tags || {};
    tags.region = process.env.AWS_REGION;
    tags.runtime = RUNTIME;
    tags.coldStart = coldStart;

    const event = {
      apikey: this.APIKey,
      type: 'invocation',
      timestamp: Date.now(),
      provider: 'aws',
      functionId: invocation.functionId,
      duration: invocation.duration,
      memory: invocation.memory,
      requestId: invocation.requestId,
      tags: Object.keys(tags).map(name => ({
        name,
        value: tags[name].toString(),
      })),
    };

    if (invocation.errorMessage) {
      event.error = {
        message: invocation.errorMessage,
        stacktrace: invocation.errorStackTrace,
      };
    }

    const req = https.request(this.requestOptions, () => {
      callback();
    });
    req.setTimeout(TIMEOUT_MS, req.abort);
    req.on('error', callback);
    req.end(JSON.stringify(event));

    coldStart = false;
  }

  trackHandler(handler, tags) {
    // bind `this` to variable `sdk` so it can be used inside the callback
    const sdk = this;

    return (event, context, cb) => {
      const createInvocationInfo = (startAt) => ({
        functionId: context.invokedFunctionArn,
        duration: new Date().getTime() - startAt.getTime(),
        memory: parseInt(context.memoryLimitInMB, 10),
        provider: 'aws',
        requestId: context.invokeid,
        tags,
      });

      const startAt = new Date();

      // wrap the original callback so the invocation info can be sent before
      // returning the result to the callee
      const callback = function callback() {
        const invocationInfo = createInvocationInfo(startAt);
        sdk.sendInvocation(invocationInfo, () => {
          cb.apply(this, arguments); // eslint-disable-line prefer-rest-params
        });
      };

      // run the provided handler in a try/catch to track errors
      try {
        handler.apply(this, [event, context, callback]);
      } catch (error) {
        const invocationInfo = createInvocationInfo(startAt);
        invocationInfo.errorMessage = error.message;
        invocationInfo.errorStackTrace = error.stack;
        sdk.sendInvocation(invocationInfo, () => {
          throw error;
        });
      }
    };
  }
}

module.exports = (key, options) => new SDK(key, options);
